UNIT mTEmpfaenger;

interface

uses
  mTNetzHW, classes, windows, sysUtils, uUtil;

type
   TEreignis = procedure of object;

   TEmpfaenger = CLASS(TThread)

     // weitere Attribute
     private
        rahmen : array[0..20] of boolean; // Bits als boolean
        crc    : byte;    // 4-Bit-CRC
        dat    : byte;    // 8-Bit-Daten
        eadr   : byte;    // 4-Bit-eigene Adresse
        aadr   : byte;    // 4-Bit-Absenderadresse
        zadr   : byte;    // 4-Bit-Zieladresse
        alles  : boolean;
        baud   : integer;
        puffer : string;
        HW     : TNetzHW;
     public
        OnChange     : TEreignis;

     // weitere Methoden
     public
        constructor create; virtual;
        procedure entpacke_Rahmen;
        procedure execute; override;
        procedure SetBaud (pBaud: integer); virtual;
        procedure SetCrc (pCrc: byte); virtual;
        procedure SetDat (pDat: byte); virtual;
        procedure SetEadr (pEadr: byte); virtual;
        procedure SetPuffer (pPuffer: string); virtual;
        procedure SetZadr (pZadr: byte); virtual;
        procedure SetzeHW (pHW: TNetzHW); virtual;
        procedure SetAlles(pW : boolean); virtual;
        function GetBaud : integer; virtual;
        function GetCrc : byte; virtual;
        function GetDat : byte; virtual;
        function GetEadr : byte; virtual;
        function GetPuffer : string;
        function GetZadr : byte; virtual;
        function GetAlles : boolean;
        function CRC_OK : boolean;
   end;

implementation

//+---------------------------------------------------------------------
//|         TEmpfaenger: Methodendefinition
//+---------------------------------------------------------------------


//-------- create (public) ---------------------------------------------
constructor TEmpfaenger.create;
begin
  inherited create(true);
  FreeOnTerminate := true;
  puffer := '';
end;

//-------- entpacke_Rahmen (public) ------------------------------------
procedure TEmpfaenger.entpacke_Rahmen;
var
  i,h1,h2,h3,stellenwert : byte;
begin
  h1 := 0; h2 := 0; h3 := 0; stellenwert := 8;
  for i := 1 to 4 do
  begin
    if rahmen[i] then h1 := h1 + stellenwert;
    if rahmen[4+i] then h2 := h2 + stellenwert;
    if rahmen[16+i] then h3 := h3 + stellenwert;
    stellenwert := stellenwert div 2;
  end;
  zadr := h1; aadr := h2; crc := h3;
  h1 := 0; stellenwert := 128;
  for i := 1 to 8 do
  begin
    if rahmen[8+i] then h1 := h1 + stellenwert;
    stellenwert := stellenwert div 2;
  end;
  dat := h1;
end;

function TEmpfaenger.CRC_OK : boolean;
var
  nachricht : word;
begin
  nachricht := zadr; nachricht := nachricht shl 4;
  nachricht := nachricht+aadr; nachricht := nachricht shl 8;
  nachricht := nachricht+dat;
  result := (crc = crc4(nachricht));
end;

//-------- execute (public) --------------------------------------------
procedure TEmpfaenger.execute;
var
  bitZeit,i : integer;
begin
  while not Terminated do
  begin
    // Baudrate berechnen
    bitZeit := round(1000/baud);
    // Puffer lschen
    puffer := '';
    // auf StartBit warten
    while HW.GetEin <> true do sleep(0);
    // abwarten bis Mitte Bit 1
    sleep(bitZeit + bitZeit div 2);
    // Bits abtasten
    for i := 1 to 20 do
    begin
      rahmen[i] := HW.GetEin;
      sleep(bitZeit);
    end;
    // StoppBit
    sleep(bitZeit);
    // Rahmen auspacken
    entpacke_Rahmen;
    // CRC-Prfung
    if crc_OK
    then
      if (zadr=eadr) or (zadr=15) or alles
      then
        puffer := chr(dat)
      else
    else
      puffer := chr(dat)+'~';
    // Auslsen des OnChange-Ereignisses
    if Assigned(OnChange) then synchronize(OnChange);
  end;
end;

//-------- SetBaud (public) --------------------------------------------
procedure TEmpfaenger.SetBaud (pBaud: integer);
begin
  baud := pBaud;
end;

//-------- SetCrc (public) ---------------------------------------------
procedure TEmpfaenger.SetCrc (pCrc: byte);
begin
  crc := pCrc;
end;

//-------- SetDat (public) ---------------------------------------------
procedure TEmpfaenger.SetDat (pDat: byte);
begin
  dat := pDat;
end;

//-------- SetEadr (public) --------------------------------------------
procedure TEmpfaenger.SetEadr (pEadr: byte);
begin
  eadr := pEadr;
end;

//-------- SetPuffer (public) ------------------------------------------
procedure TEmpfaenger.SetPuffer (pPuffer: string);
begin
  puffer := pPuffer;
  if Assigned(OnChange) then OnChange;
end;

//-------- SetZadr (public) --------------------------------------------
procedure TEmpfaenger.SetZadr (pZadr: byte);
begin
  zadr := pZadr;
end;

//-------- SetzeHW (public) --------------------------------------------
procedure TEmpfaenger.SetzeHW (pHW: TNetzHW);
begin
  HW := pHW;
end;

procedure TEmpfaenger.SetAlles(pW : boolean);
begin
  alles := pW;
end;

//-------- GetBaud (public) --------------------------------------------
function TEmpfaenger.GetBaud : integer;
begin
  result := baud;
end;

//-------- GetCrc (public) ---------------------------------------------
function TEmpfaenger.GetCrc : byte;
begin
  result := crc;
end;

//-------- GetDat (public) ---------------------------------------------
function TEmpfaenger.GetDat : byte;
begin
  result := dat;
end;

//-------- GetEadr (public) --------------------------------------------
function TEmpfaenger.GetEadr : byte;
begin
  result := eadr;
end;

//-------- GetPuffer (public) ------------------------------------------
function TEmpfaenger.GetPuffer : string;
begin
  result := puffer;
end;

//-------- GetZadr (public) --------------------------------------------
function TEmpfaenger.GetZadr : byte;
begin
  result := zadr;
end;

function TEmpfaenger.GetAlles : boolean;
begin
  result := alles;
end;

end.
