unit uTerminal4;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, mTNetzHW, mTSender, mTEmpfaenger, StdCtrls, mmsystem;

type
  TForm1 = class(TForm)
    gbEmpf: TGroupBox; mEmpf: TMemo; bLoesche: TButton; gbSend: TGroupBox;
    bSende: TButton; mSend: TMemo; gbSettings: TGroupBox;
    lBaud: TLabel; rbCom1: TRadioButton;
    rbCom2: TRadioButton; lEAdr: TLabel; eEAdr: TEdit;
    eZAdr: TEdit; lZAdr: TLabel; cbBaud: TComboBox;
    lAktiv: TLabel;
    bStop: TButton;
    lKollision: TLabel;
    cbAlles: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bSendeClick(Sender: TObject);
    procedure bLoescheClick(Sender: TObject);
    procedure rbCom2Click(Sender: TObject);
    procedure rbCom1Click(Sender: TObject);
    procedure eEAdrChange(Sender: TObject);
    procedure eZAdrChange(Sender: TObject);
    procedure cbBaudSelect(Sender: TObject);
    procedure bStopClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure cbAllesClick(Sender: TObject);
  private
    HW         : TNetzHW;
    Send       : TSender;
    Empf       : TEmpfaenger;
    SendAktiv  : boolean;
    procedure updateView;
    procedure OnSendeStart;
    procedure OnSendeEnd;
    procedure OnKollision;
  end;

  TKollisionsAnzeige = class(TThread)
  public
    procedure AnzeigeAn;
    procedure AnzeigeAus;
    procedure execute; override;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  // Netz-Hardware
  timeBeginPeriod(1);
  HW := TNetzHW.Create;
  HW.Start(1);
  // Empfnger-Thread
  Empf := TEmpfaenger.create;
  if not SetThreadPriorityBoost(Empf.Handle,true)
  then
    showMessage('Fehler beim Abschalten des PriorityBoost'); 
  Empf.setBaud(StrToInt(cbBaud.Text));
  Empf.SetEadr(StrToInt(eEadr.Text));
  Empf.SetzeHW(HW);
  Empf.OnChange := updateView;
  Empf.Resume;
  // Sender anfangs inaktiv
  sendAktiv := false;
  lAktiv.Caption := '';
  lKollision.Caption := '';
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Empf.Terminate;
  HW.Free;
  timeEndPeriod(1);
end;

procedure TForm1.updateView;
begin
  mEmpf.Text := mEmpf.Text+Empf.GetPuffer;
end;

procedure TForm1.OnSendeStart;
begin
  SendAktiv := true;
  lAktiv.caption := 'aktiv';
end;

procedure TForm1.OnSendeEnd;
begin
  SendAktiv := false;
  lAktiv.caption := '';
end;

procedure TForm1.OnKollision;
var
  Anzeige : TKollisionsAnzeige;
begin
  Anzeige := TKollisionsAnzeige.Create(true);    Anzeige.FreeOnTerminate := true;
  Anzeige.Resume;
end;

procedure TForm1.bSendeClick(Sender: TObject);
begin
  Send := TSender.create;
  if not SetThreadPriorityBoost(Send.Handle,true)
  then
    showMessage('Fehler beim Abschalten des PriorityBoost');
  Send.setBaud(StrToInt(cbBaud.Text));
  Send.SetZadr(StrToInt(eZAdr.text));
  Send.SetEadr(StrToInt(eEAdr.text));
  Send.SetzeHW(HW);
  Send.SetSendeStr(mSend.text);
  Send.OnStart := OnSendeStart;
  Send.OnEnd := OnSendeEnd;
  Send.OnColl := OnKollision;
  Send.Resume;
end;

procedure TForm1.bLoescheClick(Sender: TObject);
begin
  mEmpf.Text := '';
end;

procedure TForm1.rbCom2Click(Sender: TObject);
begin
  HW.Stop; HW.Start(2);
end;

procedure TForm1.rbCom1Click(Sender: TObject);
begin
  HW.Stop; HW.Start(1);
end;

procedure TForm1.cbBaudSelect(Sender: TObject);
begin
  if SendAktiv then Send.setBaud(StrToInt(cbBaud.Text));
  Empf.setBaud(StrToInt(cbBaud.Text));
end;

procedure TForm1.eEAdrChange(Sender: TObject);
begin
  if SendAktiv then Send.SetEadr(StrToInt(eEAdr.Text));
  Empf.SetEadr(StrToInt(eEAdr.Text));
end;

procedure TForm1.eZAdrChange(Sender: TObject);
begin
  if SendAktiv then Send.SetZAdr(StrToInt(eZAdr.Text));
end;

procedure TForm1.bStopClick(Sender: TObject);
begin
  if SendAktiv then Send.Terminate;
end;

procedure TKollisionsAnzeige.AnzeigeAn;
begin
  Form1.lKollision.caption := 'Kollision';
end;

procedure TKollisionsAnzeige.AnzeigeAus;
begin
  Form1.lKollision.caption := '';
end;

procedure TKollisionsAnzeige.execute;
begin
  synchronize(AnzeigeAn);
  sleep(100);
  windows.beep(500,1);
  synchronize(AnzeigeAus);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  OnKollision;
end;

procedure TForm1.cbAllesClick(Sender: TObject);
begin
  if cbAlles.Checked
  then
    Empf.SetAlles(true)
  else
    Empf.SetAlles(false);
end;

end.
